from lztools.click import proper_command, proper_group

from lztools.ResourceManager import out_path

@proper_group()
def main():
    pass

@proper_command()
def has_output():
    print(out_path.stat().st_size > 0)


from builtins import list as blist

import click
from lztools.click import proper_group, proper_command
from lztools.git import list_files, load_file, save_file

from lztools import ResourceManager
from lztools.ResourceManager import resources_path

@proper_group()
def main():
    """Template bash command -h help text by Laz, ᒪᗩᘔ, ㄥ卂乙, ןɐz, lคz, ℓДՀ, լᕱᏃ, Նคઽ, ﾚﾑ乙"""

@proper_command()
@click.option("-f", "--force", "--override", is_flag=True, default=False)
def initialize(override):
    ResourceManager.ensure_initialized(override=override)

@proper_command()
def list():
    files = blist(list_files(str(resources_path)))
    for k, v in enumerate(files):
        print(f'{k}:\t{v.path}')

@proper_command()
def load():
    files = blist(list_files(str(resources_path)))
    for k, v in enumerate(files):
        print(f'{k}:\t{v.path}')
    id = int(input("File #:\n"))
    load_file(str(resources_path), files[id])

@proper_command()
@click.argument("PATH", default=click.get_text_stream('stdin'))
def save(path):
    save_file(str(resources_path), path)


from datetime import datetime
from subprocess import call

import sh
from lztools.git import GitFileData

from lztools import ResourceManager
from lztools.ResourceManager import resources_path as vault_path

_vault_repo = sh.git.bake(_cwd=vault_path)

def add_file(file:str, remove_original=False):
    ResourceManager.ensure_initialized()
    cmd = "mv" if remove_original else "cp"
    call([cmd, file, str(vault_path) + "/" + file])
    _vault_repo.add(file)
    _vault_repo.commit(file, "-m", f'"{datetime.now()}"')
    _vault_repo.push()

def load_file(file:GitFileData):
    """git checkout HEAD -- {path}"""
    try:
        _vault_repo.checkout("master", "--", file.path)
        # print("Successfully loaded: {}".format(file.path))
    except Exception as e:
        raise Exception("There was an error checking out file: {}\n{}".format(file.path, e))
    call(["cp", "-vr", str(vault_path)+"/" + file.path, "."])

def list_files(filter=None, branch="HEAD"):
    """git ls-tree -r -t HEAD/{branch} --name-only"""
    files = _vault_repo('ls-tree', "-r", "-t", branch).stdout.decode("utf8").strip().splitlines()
    for data in files:
        split, file = data.split("\t")
        if filter is None or filter in file:
            permissions, type, hash = split.split(" ")
            yield GitFileData(permissions, type, hash, file)

def select_file():
    pairs = {}
    for k, v in enumerate(list_files(str(vault_path))):
        print(f'{k}:\t{v.path}')
        pairs[k] = v
    id = int(input("File #:\n"))
    return pairs[id]

def clone_repo(url, name=None):
    args = ["git", "clone", url]
    if name is not None:
        args.append(name)
    call(args)


from pathlib import Path
from subprocess import call

from lztools.TempPath import TempPath
from lztools.git import clone_repo

from lztools.shared_resources import repos
from lztools.shared_resources.strings import data_loader_bash_script

_user_home = Path.home().absolute()
main_path = _user_home .joinpath(".lztools").absolute()
scripts_path = main_path.joinpath("scripts")
loader_path = scripts_path.joinpath("data_loader")
data_path = main_path.joinpath("data")
out_path = data_path.joinpath("out_data")
resources_path = main_path.joinpath("resources")
bashrc_path = _user_home.joinpath(".bashrc")

def ensure_initialized(override=False):
    if override:
        call(["rm", "-rf", str(main_path)])
    if not main_path.exists():
        main_path.mkdir()
        scripts_path.mkdir()
        loader_path.touch()
        data_path.mkdir()
        out_path.touch()
        loader_path.write_text(data_loader_bash_script.format(out_path.absolute()))

        with TempPath(main_path.absolute()):
            clone_repo(repos.Resources)

        data = bashrc_path.read_text()
        if "PROMPT_COMMAND" not in data:
            marker = "# ∙∙∙∙∙·▫▫ᵒᴼᵒ▫ₒₒ▫ᵒᴼᵒ▫ₒₒ▫ᵒᴼᵒ☼)===> OTHER END <===(☼ᵒᴼᵒ▫ₒₒ▫ᵒᴼᵒ▫ₒₒ▫ᵒᴼᵒ▫▫·∙∙∙∙∙"
            data = data.replace(f"\n\n{marker}", f"\nPROMPT_COMMAND='source {str(loader_path)}'\n\n{marker}")
            bashrc_path.write_text(data)